using System.Collections;
using System.Collections.Generic;
using Godot;
using System;



namespace Rokojori
{
  [Tool]
  [GlobalClass]
  public partial class ClipMapPlaneMeshType:__PlaneMeshType__
  { 
    [Export]
    public float maxCellSize = 100;

    [Export]
    public float minCellSize = 10;

    [Export]
    public Curve cellSizeOverDistance = MathX.Curve( 0, 1 );
    
      
    public override MeshGeometry GetMeshGeometry( float sizeX, float sizeZ )
    {
      var xUnits = Mathf.Ceil( ( sizeX / 2 ) / maxCellSize ) * 2;
      var zUnits = Mathf.Ceil( ( sizeZ / 2 ) / maxCellSize ) * 2;

      var mg = new MeshGeometry();

      var size = new Vector2( xUnits * maxCellSize, zUnits * maxCellSize );
      var offset = size / -2f;
      var maxDistance = offset.Length();

      this.LogInfo( xUnits, zUnits );
      
      for ( int i = 0; i < xUnits; i++ )
      {
        for ( int j = 0; j < zUnits; j++ )
        {
          var c = offset + new Vector2( i * maxCellSize, j * maxCellSize );
          var s = c.Length() / maxDistance;

          if ( cellSizeOverDistance != null )
          {
            s = cellSizeOverDistance.Sample( s );
          }

          s = Mathf.Lerp( minCellSize, maxCellSize, s );

          Add( mg, c, s );
      
        } 
      }



      return mg;
    }

    void Add( MeshGeometry mg, Vector2 center, float cellSize )
    {
      var numDivisions = Mathf.CeilToInt( maxCellSize / cellSize );

      var sectionMG = MeshGeometry.CreateFromUVFunction( 
        ( uv ) =>
        {
          var pose = new Pose();

          return pose;

        },
        numDivisions, numDivisions
      );

      mg.Add( sectionMG );
    }
  }
}